' ESP8266 experiments
' default baud rate is 9600 but can increase if needed to 115200 for large websites
' ESP8266 input is from P8, reset is P9 and the output goes to P10

CON

  _clkmode = xtal1 + pll16x                '
  _xinfreq = 5_000_000
  
  ledPIN   = 26
           
OBJ

  esp8266       : "FullDuplexSerial_rr004"    
  terminal      : "FullDuplexSerial_rr004"
  delay         : "Timing"
  strings       : "ASCII0_STREngine.spin"
        
VAR

   byte string1[256] ' general purpose string buffer
   byte string2[256] ' general purpose string buffer
   byte string3[256] ' general purpose string buffer
  
PUB main 

  esp8266.start(10,8,%0000,9600)                            ' rxpin,txpin,mode,baudrate
  terminal.start(31,30,%0000,115200)                        ' propeller terminal F12, 115200 baud
  terminal.str(string("ESP8266 test program",13,10))


   Reset                                                ' reset the module
   flashled
   commandAT                                            ' send a wakeup "AT"
   flashled
   commandCWMODE                                        ' set mode to 3 (?)
   flashled
   commandCWLAP                                         ' list access points, resets the module if then use connectwifi
   flashled
    ' so reset anyway

   Reset                                                ' reset the module
   commandCWMODE                                        ' set mode to 3 (?)
   flashled
   connectWifi                                          ' connect to wifi
   flashled
   displayIP                                            ' display assigned IP address
   flashled
   commandCIPMUX                                        ' set to 1 (?)
   flashled
   'connectGoogle
   thingspeakSend
   'connectParallax                                     ' does not respond...

   'XivelySend                                          ' requires xively account...

   RunTerminal                                          ' fall through to manual terminal
   repeat ' do nothing

PUB Reset | i,c

  dira[9]~~                                             ' reset pin is an output
  dira[ledPin]~~                                        ' pin ledPin has a diagnostic led
  outa[9]~                                              ' set low
  LedHigh                                               ' and flash led
  delay.pause1ms(200)                                   ' small delay
  outa[9]~~                                             ' set high
  LedLow
  repeat i from 0 to 1000
    delay.pause1ms(1)
    c:= esp8266.rxcheck
    if c<> -1
      terminal.tx(c)                                    'echo the signon message
      
  

PUB RunTerminal | c                     ' use F12, type AT commands to test module
  
  repeat
    c := esp8266.rxcheck                                ' -1 if no data, 0-255 if there is a byte
    if c <> -1                                          ' c contains the byte
       terminal.tx(c)                                   ' send to terminal
    c := terminal.rxcheck
    if c <> -1
      esp8266.tx(c)
      if c == 13
        esp8266.tx(10)                                  ' module needs cr and lf

PUB flashled

   ledhigh
   delay.pause1ms(30)
   ledlow

PUB LedHigh

  outa[ledPin]~~

PUB LedLow

  outa[ledPin]~

PUB delayOK | i,delayabort,c,d,e ' times out after 8 seconds or if no OK. Sometimes takes 4 to 5 serial.find loops ie 4-5 seconds to respond

  i := 0
  d := -1
  e := -1
  delayabort := false
  repeat
    c:= esp8266.rxcheck
    if c<>-1
      terminal.tx(c) ' echo to terminal
      if (c== "O") and (d == -1)
        d := c
      if (c == "K") and (e == -1)
        e := c  
    delay.pause1ms(10)
    i := i+1
    if (d == "O") and (e == "K")
      delayabort := true
  until (i>800) or (delayabort == true)
  return delayabort

PUB gobbleBytes(milliseconds) | i,c

  repeat i from 0 to milliseconds
    c := esp8266.rxcheck
    if c <>-1
      terminal.tx(c)
    delay.pause1ms(1)  

   
PUB dumpESPbuffer | c    ' flush buffer and dump to terminal
                         
  repeat
     c:= esp8266.rxcheck
     if c <>-1
       terminal.tx(c)
  until  c == -1

PUB commandAT

  esp8266.rxflush ' flush the buffer  
  esp8266.str(string("AT",13,10))
  if (delayOK == true)
    'terminal.str(string("AT working",13,10))
  else
    terminal.str(string("No reply",13,10))

PUB commandCWMODE

   terminal.str(string(13,10))    ' new line
   esp8266.rxflush ' flush the buffer
   esp8266.str(string("AT+CWMODE=3",13,10))
   delay.pause1ms(200)
   dumpESPbuffer       ' returns "no change" if ok, rather than OK

PUB commandCWLAP' list access points
                 
  esp8266.rxflush ' flush the input buffer
  esp8266.str(string("AT+CWLAP",13,10))
  gobblebytes(4000) ' delayok is not fast enough, misses characters.


PUB connectWiFi

' syntax is AT+CWJAP="myssid","mypassword"
  ' and if no password then use ""
  strings.copy(string("AT+CWJAP=",34,"GatorWireless",34,",",34,"mickey10",34,13,10),@string1) ' 34 is double quotes, the "" is for the password
  ' terminal.str(@string1)  ' print to terminal
  esp8266.str(@string1)   ' print to module
  if (delayOK == true)
    terminal.str(string(13,10,"Connected to wifi",13,10))
  else
    terminal.str(string(13,10,"Not connected to wifi",13,10))

PUB displayIP ' a useful way to see actually connected
              
  terminal.str(string("Assigned IP address(es)",13,10)) 
  esp8266.str(string("AT+CIFSR",13,10))
  delayOK

PUB commandCIPMUX

   esp8266.str(string("AT+CIPMUX=1",13,10))
   delayOK

PUB commandCIPclose' close port zero
                   
   esp8266.str(string("AT+CIPCLOSE=0",13,10))' close the connection
   delayOK  

PUB connectGoogle ' tests if the internet is alive
                  
  terminal.str(string(13,10,"Test google home page",13,10))
  strings.copy(string("AT+CIPSTART=0,",34,"TCP",34,",",34,"www.google.com",34,",80",13,10),@string1)  ' source,dest
  esp8266.str(@string1) ' google changes to google in your local country
  delayOK   ' will come back with OK then Linked
  strings.copy(string("GET / HTTP/1.1",13,10,"Host:www.google.com:80",13,10,13,10),@string1)
  commandstring1 ' send this string to the website
  commandCIPclose
  

PUB commandstring1 | n ' send string1 to a website
                       
  strings.copy(string("AT+CIPSEND=0,"),@string2)  ' source,dest
  n := strings.len(@string1) ' find the length of string 1
  strings.copy(strings.integertodecimal(n,3),@string3) ' convert to a string with leading zero padding
  strings.plus(@string3,@string2) ' source,dest
  strings.plus(string(13,10),@string2) ' add crlf
  esp8266.str(@string2) ' port to send to, number of bytes
  gobblebytes(1000)     ' wait for the > prompt to come back, should be quick
  esp8266.str(@string1) ' print the string to send
  gobblebytes(4000) ' should get some text back

PUB connectParallax

  terminal.str(string(13,10,"Test parallax home page",13,10))
  strings.copy(string("AT+CIPSTART=0,",34,"TCP",34,",",34,"www.parallax.com",34,",80",13,10),@string1)  ' source,dest
  esp8266.str(@string1) 
  delayOK               ' will come back with OK then Linked
                        
  strings.copy(string("GET / HTTP/1.1",13,10,"Host:www.parallax.com:80",13,10,13,10),@string1)
  commandstring1        ' send this string to the website
  commandCIPclose

PUB thingspeakSend | n

  ' URL: "https://thingspeak.com/channels/51149"
  ' Write API Key: 6HNG51JEOWN24FVK
  ' 
  ' https://api.thingspeak.com/update?key=6HNG51JEOWN24FVK&field1=0
  ' 
  terminal.str(string(13,10,"Test thingspeak send data",13,10))

  ' AT+CIPSTART=4,"TCP","184.106.153.149",80
  strings.copy(string("AT+CIPSTART=0,",34,"TCP",34,",",34,"api.thingspeak.com",34,",80",13,10),@string1)  ' source,dest

  esp8266.str(@string1) 
  delayOK               ' will come back with OK then Linked

  strings.copy(string("GET /update?key=6HNG51JEOWN24FVK&field1=3.3",13,10,13,10),@string1)
  commandstring1        ' send this string to the website
  commandCIPclose

PUB XivelySend | n

    strings.copy(string("AT+CIPSTART=0,",34,"TCP",34,",",34,"api.xively.com",34,",80",13,10),@string1)  
    esp8266.str(@string1) 
    delayOK   ' will come back with OK then Linked
    gobblebytes(500)
    
    strings.copy(string("Sensor1,75"),@string3)     ' the sensor value to change, and the value
    strings.copy(string("PUT /v2/feeds/"),@string1) ' build the string to send
    strings.plus(string("970253233"),@string1)      ' add the feed id
    strings.plus(string(".csv HTTP/1.1",13,10,"Host: api.xively.com",13,10,"X-ApiKey: "),@string1)
    strings.plus(string("shsCNFtxuGELLZx8ehqglXAgDo9lkyBam5Zj22p3g3urH2FM"),@string1)
    strings.plus(string(13,10,"User-Agent: "),@string1)
    strings.plus(string("Arduino1"),@string1)       ' name of the project
    strings.plus(string(13,10,"Content-Length: "),@string1)
    n := strings.len(@string3)                      ' find the length of string3 with the sensor name and value
    strings.copy(strings.integertodecimal(n,3),@string2) ' get the length as a string
    strings.plus(@string2,@string1)                 ' add the length
    strings.plus(string(13,10,"Content-Type: text/csv",13,10,"Connection: close",13,10,13,10),@string1) 
    strings.plus(@string3,@string1)                 ' the actual data to send
    strings.plus(string(13,10,13,10),@string1)      ' final crlfs
                                                    ' 
    commandstring1                                  ' send this command, waits 4s for a reply, closes automatically


    
' ***************************************************************************************
      'printstring(@lineoftext1)                ' testing string functions
      '
      'str.left(@lineoftext1,@lineoftext2,3)    ' find leftmost characters in a string
      'printstring(@lineoftext2)
      '
      'strings.trimstring(@lineoftext1)
      'printstring(@lineoftext1)
      'strings.trimstring(@lineoftext1)
      '
      'strings.mid(@lineoftext1,@lineoftext2,4,2)   ' find middle characters in a string
      'printstring(@lineoftext2)
      'printstring(str.integertodecimal(str.len(@lineoftext2),5))
      '
      'strings.copy(string("hello"),@lineoftext1)   ' copy fixed value into a string
      'printstring(@lineoftext1)
      '
      'strings.str(@lineoftext1,55)                 ' decimal to string with no leading zeros
      'printstring(@lineoftext1)
      '
      'strings.stringfill(@lineoftext1,10,65)
      'printstring(@lineoftext1)
      '
      'strings.str(@lineoftext1,1234)                                   ' convert to string
      'strings.str(@lineoftext1,str.decimaltointeger(@lineoftext1))     ' convert back
      'printstring(@lineoftext1)
      '
      'strings.left(@lineoftext1,@lineoftext2,str.len(@lineoftext1)-4)  ' remove extension
      'printstring(@lineoftext2)                                        ' print it out         